/* eslint-disable @typescript-eslint/no-var-requires */
/*
 * Copyright 2018-2019 Fibabanka. All Rights Reserved.
 *
 * Save to the extent permitted by law, you may not use, copy, modify,
 * distribute or create derivative works of this material or any part
 * of it without the prior written consent of Fibabanka.
 * Any reproduction of this material must contain this notice.
 */

const path = require('path');
const webpack = require('webpack');
const HTMLWebpackPlugin = require('html-webpack-plugin');

const argv = require('yargs').argv;
// TODO(canerKaraman): will handle sentry
// const SentryWebpackPlugin = require('@sentry/webpack-plugin');
const BundleAnalyzerPlugin = require('webpack-bundle-analyzer').BundleAnalyzerPlugin;

const appDirectory = path.resolve(__dirname, './');
const isLocal = argv.env === 'local';
const isDev = argv.env === 'dev';

const babelLoaderOptions = {
  cacheDirectory: true,
  // Babel configuration (or use .babelrc)
  // This aliases 'react-native' to 'react-native-web' and includes only
  // the modules needed by the app.
  plugins: [
    [
      '@babel/plugin-proposal-decorators',
      {
        legacy: true,
      },
    ],
    'react-native-web',
    'react-hot-loader/babel',
  ],
  // The 'react-native' preset is recommended to match React Native's packager
  presets: [
    'module:metro-react-native-babel-preset',
    '@babel/preset-typescript',
    '@babel/preset-env',
  ],
};

// This is needed for webpack to compile JavaScript.
// Many OSS React Native packages are not compiled to ES5 before being
// published. If you depend on uncompiled packages they may cause webpack build
// errors. To fix this webpack can be configured to compile to the necessary
// `node_module`.
const babelLoaderConfiguration = {
  test: /\.(js|ts)x?$/,
  // Add every directory that needs to be compiled by Babel during the build.
  include: [
    path.resolve(appDirectory, 'index.web.js'),
    path.resolve(appDirectory, 'node_modules/react'),
    path.resolve(appDirectory, 'node_modules/react-native'),

    path.resolve(appDirectory, 'node_modules/react-native-vector-icons'),
    path.resolve(appDirectory, 'node_modules/react-native-elements'),
    path.resolve(appDirectory, 'node_modules/react-native-toast-message'),
    path.resolve(appDirectory, 'node_modules/react-native-dropdown-picker'),
    path.resolve(appDirectory, 'node_modules/react-navigation'),
    path.resolve(appDirectory, 'node_modules/react-native-ratings'),
    path.resolve(appDirectory, 'node_modules/@react-navigation/drawer'),

    
    path.resolve(appDirectory, 'node_modules/react-native-reanimated'),
    
    path.resolve(appDirectory, 'App.web.tsx'),
    path.resolve(appDirectory, 'src'),
  ],
  use: {
    loader: 'babel-loader',
    options: babelLoaderOptions,
  },
};

// This is needed for webpack to import static images in JavaScript files.

const imageLoaderConfiguration = {
  test: /\.(gif|jpe?g|png|svg|css|tff|ttf)$/,
  use: {
    loader: 'url-loader',

    options: {
      name: '[name].[ext]',
      esModule: false,
    },
  },
};

module.exports = {
  // your web-specific entry file
  entry: path.resolve(appDirectory, 'index.web.js'),

  // configures where the build ends up
  output: {
    filename: 'bundle.js',
    path: path.join(__dirname, '/build'),
  },

  // ...the rest of your config

  module: {
    rules: [babelLoaderConfiguration, imageLoaderConfiguration],
  },

  plugins: [
    new HTMLWebpackPlugin({
      inject: 'body',
      template: './public/index.html',

      filename: './index.html',
    }),
    new webpack.DefinePlugin({
      'process.env.NODE_ENV':
        isDev || isLocal
          ? JSON.stringify('development')
          : JSON.stringify('production'),
    }),
    new webpack.DefinePlugin({
      'process.env.NODE_ENV': JSON.stringify(
        process.env.NODE_ENV || 'development',
      ),
      __DEV__: process.env.NODE_ENV === 'production' || true,
    }),
    new BundleAnalyzerPlugin({
      analyzerMode: isLocal ? 'server' : 'disabled'
    }),
    new webpack.DefinePlugin({process: {env: {}}}),

    // TODO(canerKaraman): will handle sentry
    // new SentryWebpackPlugin({
    //   include: ".",
    //   ignoreFile: ".sentrycliignore",
    //   ignore: ["node_modules", "webpack.config.js"],
    //   configFile: "sentry.properties"
    // })
  ],

  resolve: {
    alias: {
      'react-native$': 'react-native-web',
      'react-native-svg': 'react-native-svg-web',
    },
    // If you're working on a multi-platform React Native app, web-specific
    // module implementations should be written in files using the extension
    // `.web.js`.
    extensions: ['.web.tsx', '.web.ts', '.web.js', '.ts', '.tsx', '.js'],
  },

  devServer: {
    historyApiFallback: true,
    contentBase: './',
    hot: true,
  },
};
