import React, { useState, useEffect } from 'react';
import { StyleSheet, View, Text, TouchableOpacity, ScrollView, KeyboardAvoidingView, Platform } from 'react-native'
import { SafeAreaView } from 'react-native-safe-area-context';
// import { Picker } from '@react-native-picker/picker';
import Toast from 'react-native-toast-message';
import FibaButton from '../components/FibaButton';
import Line from '../components/Line';
import { getOrderBoard, getLastTrade, listOrders } from '../services/order-services';
// import { TabView, SceneMap } from 'react-native-tab-view'; // sil
import ReconnectingWebSocket from 'reconnecting-websocket';
import DropDownPicker from 'react-native-dropdown-picker';
import { Icon, Badge } from 'react-native-elements'
import FibaInput from '../components/FibaInput';
import AsyncStorage from '@react-native-async-storage/async-storage';
import NumberFormat from 'react-number-format';

const ws = new ReconnectingWebSocket('ws://192.168.1.17:4000/api');



export default function OrderBoard() {

    const [index, setIndex] = useState(0);
    const [indexOrder, setOrderIndex] = useState(0);

    const [exchangeType, setType] = useState("USDTRY");
    const [exchangeType2, setType2] = useState("USDTRY");
    const [orders, setOrders] = useState({buy:[], sell:[]});
    const [customerOrders, setCustomerOrders] = useState([]);
    const [price, setPrice] = useState();

    const [open, setOpen] = useState(false);
    const [open2, setOpen2] = useState(false);
    const [exchangeTypeList, setExchangeTypeList] = useState([]);

    const [selectedAccount1, setselectedAccount1] = useState();
    const [selectedAccount2, setselectedAccount2] = useState();


    function renderBuyOrderBoard (orders: {buy: [], sell: []}) {
        return orders.buy.map((order: {amount:string, price: string}, i: number)=>{
            return <View key={i} style={{flexDirection:"row", justifyContent:"flex-end", marginBottom:2}}>
                <View style={{flex:3, flexDirection:"row", justifyContent:"flex-end", marginRight:10}}>
                    <Text style={{color:"#8A8A8A"}}>{order.amount}</Text>
                    {/* <NumberFormat  style={{color:"#8A8A8A"}} displayType={'text'} thousandSeparator={'.'} decimalSeparator={','} value={order.amount} /> */}
                </View>
                <View style={{flex:2, flexDirection:"row", justifyContent:"flex-end"}}>
                    <Text style={{color:"#84BD00"}}>{order.price}</Text>
                </View>
            </View>
        })
    }

    function renderSellOrderBoard (orders: {buy: [], sell: []}) {
        return orders.sell.map((order: {amount:string, price: string}, i: number)=>{
            return <View key={i} style={{flexDirection:"row", justifyContent:"flex-start", marginBottom:2}}>
                <View style={{flex:2, flexDirection:"row", justifyContent:"flex-start", marginRight:10}}>
                    <Text style={{color:"rgb(251, 0, 47)"}}>{order.price}</Text>
                </View>
                <View style={{flex:3}}>
                    <Text style={{color:"#8A8A8A"}}>{order.amount}</Text>
                    {/* <NumberFormat  style={{color:"#8A8A8A"}} displayType={'text'} thousandSeparator={'.'} decimalSeparator={','} value={order.amount} /> */}
                </View>
        </View>
        })
    }

    useEffect(() => {
            AsyncStorage.setItem('exchange1', exchangeType.substring(0,3))
            AsyncStorage.setItem('exchange2', exchangeType.substring(3,6))
            listOrders("customerId=1234").then(res => {
                setCustomerOrders(res)
            })
            getLastTrade().then(async res=>{
                let selectedAccount1 = await AsyncStorage.getItem('selectedAccount1')
                let selectedAccount2 = await AsyncStorage.getItem('selectedAccount2')
                setselectedAccount1(JSON.parse(selectedAccount1))
                setselectedAccount2(JSON.parse(selectedAccount2))
                let exchangeTypeList = []
                res.map((trade, i)=>{
                    exchangeTypeList.push({value: trade.instrument, label: trade.instrument, price: trade.price})
                })
                setExchangeTypeList(exchangeTypeList)
                setPrice(exchangeTypeList[0].price)
                setType(exchangeTypeList[0].value)
                setType2(exchangeTypeList[0].value)
                getOrderBoard(exchangeType).then(res=>{
                    console.log(res)
                    setOrders(res)}
                )
            })
    
      }, []);

    ws.onmessage = async (e) => {
        let message = JSON.parse(e.data)
        if(message.msg.s == "board"){
            setOrders(message.msg.data[0])
            listOrders("customerId=1234").then(res => {
                setCustomerOrders(res)
            })
        }
        if(message.msg.s == "lastTrade"){
            exchangeTypeList.map((exchange)=>{
                if(message.msg.data.instrument == exchange.value){
                    exchange.price = message.msg.data.instrument;
                }
            })
        }
    };

    return (
        <SafeAreaView style={styles.container}>
            <Toast ref={(ref) => Toast.setRef(ref)} />
            <View style={[styles.topArea, { alignItems:"flex-end", paddingBottom:20,
                borderBottomColor:"rgba(190, 190, 191, 0.51)", borderBottomWidth:0.5,
                backgroundColor: '#fff',
                shadowColor: '#000',
                shadowOffset: { width: 1, height: 1 },
                shadowOpacity:  0.2,
                shadowRadius: 16,
                elevation: 10, height: 85}]}>
                <View style={{flex:2}}>
                </View>
                <View style={{flex:2, alignItems:"center"}}><Text>P2P FX Market</Text></View>
                <TouchableOpacity style={{flex:2, alignItems:"flex-end", marginRight:20}}><Icon name='account-balance-wallet' color='black' /></TouchableOpacity>
            </View>
            <ScrollView style={{flex:11}}>
                <View style={{flexDirection:"row", justifyContent:"space-around", marginBottom: 20, marginTop:30, flex:1, borderBottomColor: "rgb(238, 238, 238)", borderBottomWidth:1}}>
                    <TouchableOpacity style={index == 0 ? {borderBottomColor:"rgb(6, 84, 182)", justifyContent:"flex-end", paddingLeft:20, paddingRight:20, alignItems:"center", borderBottomWidth:2, marginLeft:50} : {borderBottomColor:"rgb(6, 84, 182)", justifyContent:"flex-end", alignItems:"center", marginLeft:50, paddingLeft:20, paddingRight:20}} onPress={()=>setIndex(0)}>
                        <Text style={{color: index == 0 ? "rgb(6, 84, 182)" : "#8A8A8A"}}>AL/SAT</Text>
                    </TouchableOpacity>
                    <TouchableOpacity style={index == 1 ? {borderBottomColor:"rgb(6, 84, 182)", justifyContent:"flex-end", borderBottomWidth:2, paddingLeft:20, paddingRight:20, alignItems:"center", marginRight:50} : {borderBottomColor:"rgb(6, 84, 182)", justifyContent:"flex-end", alignItems:"center", marginRight:50, paddingLeft:20, paddingRight:20}} onPress={()=>setIndex(1)}>
                        <Text style={{color: index == 1 ? "rgb(6, 84, 182)" : "#8A8A8A"}}>Emirler</Text>
                    </TouchableOpacity>
                </View>
                <View style={[styles.topArea, {backgroundColor: "white", borderRadius:10, marginLeft:20, marginRight:20, height: 40}]}>
                    <View style={{flexDirection: "row"}}>
                        <DropDownPicker
                        style={{flexDirection:"row", width:120, borderWidth:0, height:40}}
                        open={open}
                        value={exchangeType}
                        items={exchangeTypeList}
                        setOpen={setOpen}
                        setValue={setType}
                        placeholder="Seçiniz"
                        showTickIcon={false}
                        dropDownContainerStyle={{
                            borderRadius:0,
                            marginTop: 20,
                            width: 120
                        }}
                        />
                    </View>
                    <Text style={{marginLeft:-250, color:"rgb(251, 0, 47)", fontWeight:"bold"}}>{price}</Text>
                </View>
                <View style={{zIndex:-5, flexDirection:"row", justifyContent:"space-around", margin:20, marginLeft:40, marginRight:40, backgroundColor: "white", borderRadius:10, height: 40, alignItems:"center"}}>
                    <View><Text style={{color:"rgb(6, 84, 182)"}}>Bakiye</Text></View>
                    <View style={{flexDirection:"row"}}>
                        <Text style={{color:"rgb(51, 51, 51)"}}>{selectedAccount1 ? selectedAccount1.availableBalance : "-"}</Text>
                        <Text style={{marginLeft:5, color:"rgb(251, 0, 47)"}}>{exchangeType.substring(0,3)}</Text>
                    </View>
                    <View style={{flexDirection:"row"}}>
                        <Text style={{color:"rgb(51, 51, 51)"}}>{selectedAccount2 ? selectedAccount2.availableBalance : "-"}</Text>
                        <Text style={{marginLeft:5, color:"rgb(251, 0, 47)"}}>{exchangeType.substring(3,6)}</Text>
                    </View>
                </View>
                <View style={{flexDirection: "row", margin:20, marginLeft:40, marginRight:40, flex:1}}>
                    <FibaInput placeholder={exchangeType} style={{ width:"100%"}} placeholderTextColor={"rgb(51, 51, 51)"}></FibaInput>
                </View>
                {index == 0 ?
                <View style={{flex:10}}>

                <View style={{flexDirection:"row", justifyContent:"space-between", marginLeft:40, marginRight:40}}>
                    <FibaButton placeholder="Al" textColor="white" style={{flex:1, backgroundColor: "rgb(128, 190, 13)"}}></FibaButton>
                    <FibaButton placeholder="Sat" textColor="white" style={{flex:1, backgroundColor: "rgb(251, 0, 47)"}}></FibaButton>
                </View>
                <View style={{flexDirection:"column", marginLeft:20, marginRight:20, marginTop:20}}>
                    <View style={{flexDirection: "row", marginRight:10}}>
                        <View style={{flex:1, marginRight: 20}}>
                            <View style={{flexDirection:"row", justifyContent:"flex-end"}}>
                                <View style={{flex:4, flexDirection:"row", justifyContent:"flex-end", marginRight:10}}>
                                    <Text style={{color:"#8A8A8A", fontSize:13}}>Tutar</Text>
                                </View>
                                <View style={{flex:3, flexDirection:"row", justifyContent:"flex-end"}}>
                                    <Text style={{color:"#8A8A8A", fontSize:13}}>Alış(TRY)</Text>
                                </View>
                            </View>
                        </View>
                        <View style={{flex:1}}>
                            <View style={{flexDirection:"row", justifyContent:"flex-end"}}>
                                <View style={{flex:3, flexDirection:"row", justifyContent:"flex-start", marginRight:10}}>
                                    <Text style={{color:"#8A8A8A", fontSize:13}}>Satış(TRY)</Text>
                                </View>
                                <View style={{flex:4}}>
                                    <Text style={{color:"#8A8A8A", fontSize:13}}>Tutar</Text>
                                </View>
                            </View>
                        </View>
                    </View>
                </View>   
                <View style={{flexDirection:"row", justifyContent:"space-between", marginLeft:20, marginRight:20}}>
                    <Line style={{borderColor:"rgb(130, 192, 24)"}}></Line>
                    <Line style={{borderColor:"rgb(251, 0, 47)"}}></Line>
                </View>
                <View style={{flexDirection:"column", marginLeft:20, marginRight: 20}}>
                    <View style={{flexDirection: "row", marginRight:10}}>
                        <View style={{flex:1, marginRight: 20}}>
                            {orders ? renderBuyOrderBoard(orders) : null}
                        </View>
                        <View style={{flex:1}}>
                            {orders ? renderSellOrderBoard(orders) : null}
                        </View>
                    </View>
                </View>   
                </View> :
                index == 1 ?
                <View style={{flex:10}}>
                    <View style={{backgroundColor: "white"}}>
                        <View style={{flexDirection:"row", justifyContent:"space-around", marginBottom: 20, marginTop:10, borderBottomColor: "rgb(238, 238, 238)", borderBottomWidth:1}}>
                            <TouchableOpacity style={indexOrder == 0 ? {flexDirection:"row", borderBottomColor:"rgb(6, 84, 182)", justifyContent:"flex-end", alignItems:"center", borderBottomWidth:2, marginLeft:20, paddingLeft:10, paddingRight:10} : {flexDirection:"row", borderBottomColor:"rgb(6, 84, 182)", justifyContent:"flex-end", alignItems:"center", marginLeft:20, paddingLeft:10, paddingRight:10}} onPress={()=>setOrderIndex(0)}>
                                <Text style={{color: indexOrder == 0 ? "rgb(6, 84, 182)" : "#8A8A8A"}}>Aktif</Text>
                                <Badge status="warning" containerStyle={{ position: 'absolute', top: -4, right: -10 }} value={"8"}></Badge>
                            </TouchableOpacity>
                            <TouchableOpacity style={indexOrder == 1 ? {flexDirection:"row", borderBottomColor:"rgb(6, 84, 182)", justifyContent:"flex-end", borderBottomWidth:2, alignItems:"center", paddingLeft:10, paddingRight:10} : {flexDirection:"row", borderBottomColor:"rgb(6, 84, 182)", justifyContent:"flex-end", alignItems:"center", paddingLeft:10, paddingRight:10}} onPress={()=>setOrderIndex(1)}>
                                <Text style={{color: indexOrder == 1 ? "rgb(6, 84, 182)" : "#8A8A8A"}}>Gerçekleşmiş</Text>
                                {/* <Badge status="warning" containerStyle={{ position: 'absolute', top: -4, right: -10 }} value={"8"}></Badge> */}
                            </TouchableOpacity>
                            <TouchableOpacity style={indexOrder == 2 ? {flexDirection:"row", borderBottomColor:"rgb(6, 84, 182)", justifyContent:"flex-end", borderBottomWidth:2, alignItems:"center", marginRight:20, paddingLeft:10, paddingRight:10} : {flexDirection:"row", borderBottomColor:"rgb(6, 84, 182)", justifyContent:"flex-end", alignItems:"center", marginRight:20, paddingLeft:10, paddingRight:10}} onPress={()=>setOrderIndex(2)}>
                                <Text style={{color: indexOrder == 2 ? "rgb(6, 84, 182)" : "#8A8A8A"}}>İptal Edilmiş</Text>
                                <Badge status="warning" containerStyle={{ position: 'absolute', top: -4, right: -10 }} value={"8"}></Badge>
                            </TouchableOpacity>
                        </View>
                        <View style={{flexDirection: "row"}}>
                            <View style={{flex:1}}></View>
                            <View style={{flex:1}}><Text>Fiyat</Text></View>
                            <View style={{flex:1}}><Text>Tutar</Text></View>
                            <View style={{flex:1}}><Text>İşlem</Text></View>
                        </View>
                        {indexOrder == 0 ?
                        customerOrders.map((order, i)=>{
                            if(order.status == "Open")
                            return <View style={{flexDirection: "row", alignItems: "center"}}>
                            <View style={{flex:1}}><Text style={{color:"#8A8A8A", marginLeft:20}}>{order.instrument}</Text></View>
                            <View style={{flex:1}}><Text style={{color:"#8A8A8A"}}>{order.price}</Text></View>
                            <View style={{flex:1}}><Text style={{color:"rgb(253, 180, 4)"}}>{order.bidAmount}</Text></View>
                            <View style={{flex:1}}>
                                <FibaButton textColor={"white"} placeholder={"İptal"} noicon={true} style={{height:25, backgroundColor: "rgb(251, 0, 47)"}}></FibaButton>
                            </View>
                        </View>
                        }) : indexOrder == 1 ?

                        customerOrders.map((order, i)=>{
                            if(order.status == "Completed")
                            return <View style={{flexDirection: "row", alignItems: "center"}}>
                            <View style={{flex:1}}><Text style={{color:"#8A8A8A", marginLeft:20}}>{order.instrument}</Text></View>
                            <View style={{flex:1}}><Text style={{color:"#8A8A8A"}}>{order.price}</Text></View>
                            <View style={{flex:1}}><Text style={{color:"rgb(253, 180, 4)"}}>{order.bidAmount}</Text></View>
                            <View style={{flex:1}}>
                                <View style={{height:40, justifyContent:"center"}}><Text style={{color: "#84BD00"}}>Tamamlandı</Text></View>
                            </View>
                        </View>
                        }) :
                        customerOrders.map((order, i)=>{
                            if(order.status == "Incomplete")
                            return <View style={{flexDirection: "row", alignItems: "center"}}>
                            <View style={{flex:1}}><Text style={{color:"#8A8A8A", marginLeft:20}}>{order.instrument}</Text></View>
                            <View style={{flex:1}}><Text style={{color:"#8A8A8A"}}>{order.price}</Text></View>
                            <View style={{flex:1}}><Text style={{color:"rgb(253, 180, 4)"}}>{order.bidAmount}</Text></View>
                            <View style={{flex:1}}>
                                <View style={{height:40, justifyContent:"center"}}><Text style={{color: "red"}}>İptal</Text></View>
                            </View>
                        </View>
                        })
                    }
                        
                    </View>
                </View> : null}  
            </ScrollView>
            
        </SafeAreaView>
    );
  }
  
  const styles = StyleSheet.create({
    container: {
      flex: 1,
      flexDirection:"column",
      backgroundColor: 'rgb(247, 247, 247)',
    //   alignItems: 'center',
    //   justifyContent: 'center',
    },
    topArea: {
        flexDirection: "row",
        alignItems: "center",
        justifyContent: "center",
    },
    bottomArea: {
        flex: 11,
    },
      tabBar: {
        flexDirection: 'row',
        paddingTop: 3,
      },
      tabItem: {
        flex: 1,
        alignItems: 'center',
        padding: 16,
      }
  });